@@ -19,6 +19,8 @@
  *
  */
 
+#include <stdbool.h>
+
 #include <glib.h>
 
 #define CONNMAN_API_SUBJECT_TO_CHANGE
@@ -51,11 +53,16 @@ DBusMessage *__connman_error_operation_timeout(DBusMessage *msg);
 DBusMessage *__connman_error_invalid_service(DBusMessage *msg);
 DBusMessage *__connman_error_invalid_property(DBusMessage *msg);
 
-#include <connman/types.h>
-
 int __connman_manager_init(void);
 void __connman_manager_cleanup(void);
 
+enum time_updates {
+	TIME_UPDATES_UNKNOWN = 0,
+	TIME_UPDATES_MANUAL  = 1,
+	TIME_UPDATES_AUTO    = 2,
+};
+
+enum time_updates __connman_clock_timeupdates(void);
 int __connman_clock_init(void);
 void __connman_clock_cleanup(void);
 
@@ -86,16 +93,17 @@ void __connman_agent_cancel(struct connman_service *service);
 int __connman_service_add_passphrase(struct connman_service *service,
 					const gchar *passphrase);
 typedef void (* authentication_cb_t) (struct connman_service *service,
-				connman_bool_t values_received,
+				bool values_received,
 				const char *name, int name_len,
 				const char *identifier, const char *secret,
-				gboolean wps, const char *wpspin,
+				bool wps, const char *wpspin,
 				const char *error, void *user_data);
 typedef void (* browser_authentication_cb_t) (struct connman_service *service,
-				connman_bool_t authentication_done,
+				bool authentication_done,
 				const char *error, void *user_data);
 int __connman_agent_request_passphrase_input(struct connman_service *service,
-				authentication_cb_t callback, void *user_data);
+				authentication_cb_t callback,
+				const char *dbus_sender, void *user_data);
 int __connman_agent_request_login_input(struct connman_service *service,
 				authentication_cb_t callback, void *user_data);
 int __connman_agent_request_browser(struct connman_service *service,
@@ -105,9 +113,9 @@ int __connman_agent_request_browser(struct connman_service *service,
 #include <connman/log.h>
 
 int __connman_log_init(const char *program, const char *debug,
-		connman_bool_t detach, connman_bool_t backtrace,
+		gboolean detach, gboolean backtrace,
 		const char *program_name, const char *program_version);
-void __connman_log_cleanup(connman_bool_t backtrace);
+void __connman_log_cleanup(gboolean backtrace);
 void __connman_log_enable(struct connman_debug_desc *start,
 					struct connman_debug_desc *stop);
 
@@ -134,6 +142,7 @@ int __connman_inet_modify_address(int cmd, int flags, int index, int family,
 				unsigned char prefixlen,
 				const char *broadcast);
 int __connman_inet_get_interface_address(int index, int family, void *address);
+int __connman_inet_get_interface_ll_address(int index, int family, void *address);
 
 #include <netinet/ip6.h>
 #include <netinet/icmp6.h>
@@ -143,8 +152,25 @@ typedef void (*__connman_inet_rs_cb_t) (struct nd_router_advert *reply,
 
 int __connman_inet_ipv6_send_rs(int index, int timeout,
 			__connman_inet_rs_cb_t callback, void *user_data);
+int __connman_inet_ipv6_send_ra(int index, struct in6_addr *src_addr,
+				GSList *prefixes, int router_lifetime);
 
-int __connman_refresh_rs_ipv6(struct connman_network *network, int index);
+typedef void (*__connman_inet_ns_cb_t) (struct nd_neighbor_advert *reply,
+					unsigned int length,
+					struct in6_addr *addr,
+					void *user_data);
+int __connman_inet_ipv6_do_dad(int index, int timeout_ms,
+			struct in6_addr *addr,
+			__connman_inet_ns_cb_t callback, void *user_data);
+
+typedef void (*__connman_inet_recv_rs_cb_t) (struct nd_router_solicit *reply,
+					unsigned int length, void *user_data);
+int __connman_inet_ipv6_start_recv_rs(int index,
+				__connman_inet_recv_rs_cb_t callback,
+				void *user_data, void **context);
+void __connman_inet_ipv6_stop_recv_rs(void *context);
+
+int __connman_network_refresh_rs_ipv6(struct connman_network *network, int index);
 
 GSList *__connman_inet_ipv6_get_prefixes(struct nd_router_advert *hdr,
 					unsigned int length);
@@ -193,9 +219,16 @@ int __connman_inet_rtnl_addattr_l(struct nlmsghdr *n, size_t max_length,
 int __connman_inet_rtnl_addattr32(struct nlmsghdr *n, size_t maxlen,
 			int type, __u32 data);
 
+int __connman_inet_add_fwmark_rule(uint32_t table_id, int family, uint32_t fwmark);
+int __connman_inet_del_fwmark_rule(uint32_t table_id, int family, uint32_t fwmark);
+int __connman_inet_add_default_to_table(uint32_t table_id, int ifindex, const char *gateway);
+int __connman_inet_del_default_from_table(uint32_t table_id, int ifindex, const char *gateway);
+int __connman_inet_get_address_netmask(int ifindex,
+		struct sockaddr_in *address, struct sockaddr_in *netmask);
+
 #include <connman/resolver.h>
 
-int __connman_resolver_init(connman_bool_t dnsproxy);
+int __connman_resolver_init(gboolean dnsproxy);
 void __connman_resolver_cleanup(void);
 int __connman_resolvfile_append(int index, const char *domain, const char *server);
 int __connman_resolvfile_remove(int index, const char *domain, const char *server);
@@ -213,9 +246,9 @@ GKeyFile *__connman_storage_open_service(const char *ident);
 int __connman_storage_save_service(GKeyFile *keyfile, const char *ident);
 GKeyFile *__connman_storage_load_provider(const char *identifier);
 void __connman_storage_save_provider(GKeyFile *keyfile, const char *identifier);
-gboolean __connman_storage_remove_provider(const char *identifier);
+bool __connman_storage_remove_provider(const char *identifier);
 char **__connman_storage_get_providers(void);
-gboolean __connman_storage_remove_service(const char *service_id);
+bool __connman_storage_remove_service(const char *service_id);
 
 int __connman_detect_init(void);
 void __connman_detect_cleanup(void);
@@ -322,7 +355,7 @@ void __connman_ipconfig_set_prefixlen(struct connman_ipconfig *ipconfig, unsigne
 
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig);
-connman_bool_t __connman_ipconfig_is_usable(struct connman_ipconfig *ipconfig);
+bool __connman_ipconfig_is_usable(struct connman_ipconfig *ipconfig);
 
 const char *__connman_ipconfig_method2string(enum connman_ipconfig_method method);
 const char *__connman_ipconfig_type2string(enum connman_ipconfig_type type);
@@ -357,14 +390,18 @@ const char *__connman_ipconfig_get_proxy_autoconfig(struct connman_ipconfig *ipc
 void __connman_ipconfig_set_dhcp_address(struct connman_ipconfig *ipconfig,
 					const char *address);
 char *__connman_ipconfig_get_dhcp_address(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_dhcpv6_prefixes(struct connman_ipconfig *ipconfig,
+					char **prefixes);
+char **__connman_ipconfig_get_dhcpv6_prefixes(struct connman_ipconfig *ipconfig);
 
 int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix);
 int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix);
-gboolean __connman_ipconfig_ipv6_privacy_enabled(struct connman_ipconfig *ipconfig);
+bool __connman_ipconfig_ipv6_privacy_enabled(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_ipv6_set_privacy(struct connman_ipconfig *ipconfig,
 					const char *value);
+bool __connman_ipconfig_ipv6_is_enabled(struct connman_ipconfig *ipconfig);
 
 int __connman_ipconfig_set_rp_filter();
 void __connman_ipconfig_unset_rp_filter(int old_value);
@@ -387,8 +424,17 @@ GSList *__connman_timeserver_get_all(struct connman_service *service);
 int __connman_timeserver_sync(struct connman_service *service);
 void __connman_timeserver_sync_next();
 
+enum __connman_dhcpv6_status {
+	CONNMAN_DHCPV6_STATUS_FAIL     = 0,
+	CONNMAN_DHCPV6_STATUS_SUCCEED  = 1,
+	CONNMAN_DHCPV6_STATUS_RESTART  = 2,
+};
+
+typedef void (* dhcpv6_cb) (struct connman_network *network,
+			enum __connman_dhcpv6_status status, gpointer data);
+
 typedef void (* dhcp_cb) (struct connman_network *network,
-				connman_bool_t success);
+			bool success, gpointer data);
 int __connman_dhcp_start(struct connman_network *network, dhcp_cb callback);
 void __connman_dhcp_stop(struct connman_network *network);
 int __connman_dhcp_init(void);
@@ -396,14 +442,20 @@ void __connman_dhcp_cleanup(void);
 int __connman_dhcpv6_init(void);
 void __connman_dhcpv6_cleanup(void);
 int __connman_dhcpv6_start_info(struct connman_network *network,
-				dhcp_cb callback);
+				dhcpv6_cb callback);
 void __connman_dhcpv6_stop(struct connman_network *network);
 int __connman_dhcpv6_start(struct connman_network *network,
-				GSList *prefixes, dhcp_cb callback);
+				GSList *prefixes, dhcpv6_cb callback);
 int __connman_dhcpv6_start_renew(struct connman_network *network,
-				dhcp_cb callback);
+				dhcpv6_cb callback);
 int __connman_dhcpv6_start_release(struct connman_network *network,
-				dhcp_cb callback);
+				dhcpv6_cb callback);
+int __connman_dhcpv6_start_pd(int index, GSList *prefixes, dhcpv6_cb callback);
+void __connman_dhcpv6_stop_pd(int index);
+int __connman_dhcpv6_start_pd_renew(struct connman_network *network,
+							dhcpv6_cb callback);
+int __connman_dhcpv6_start_pd_release(struct connman_network *network,
+				dhcpv6_cb callback);
 
 int __connman_ipv4_init(void);
 void __connman_ipv4_cleanup(void);
@@ -419,7 +471,7 @@ void __connman_connection_gateway_remove(struct connman_service *service,
 					enum connman_ipconfig_type type);
 int __connman_connection_get_vpn_index(int phy_index);
 
-gboolean __connman_connection_update_gateway(void);
+bool __connman_connection_update_gateway(void);
 void __connman_connection_gateway_activate(struct connman_service *service,
 					enum connman_ipconfig_type type);
 
@@ -445,19 +497,19 @@ int __connman_technology_add_device(struct connman_device *device);
 int __connman_technology_remove_device(struct connman_device *device);
 int __connman_technology_enabled(enum connman_service_type type);
 int __connman_technology_disabled(enum connman_service_type type);
-int __connman_technology_set_offlinemode(connman_bool_t offlinemode);
-connman_bool_t __connman_technology_get_offlinemode(void);
+int __connman_technology_set_offlinemode(bool offlinemode);
+bool __connman_technology_get_offlinemode(void);
 void __connman_technology_set_connected(enum connman_service_type type,
-					connman_bool_t connected);
+					bool connected);
 
 int __connman_technology_add_rfkill(unsigned int index,
 					enum connman_service_type type,
-						connman_bool_t softblock,
-						connman_bool_t hardblock);
+						bool softblock,
+						bool hardblock);
 int __connman_technology_update_rfkill(unsigned int index,
 					enum connman_service_type type,
-						connman_bool_t softblock,
-						connman_bool_t hardblock);
+						bool softblock,
+						bool hardblock);
 int __connman_technology_remove_rfkill(unsigned int index,
 					enum connman_service_type type);
 
@@ -485,7 +537,7 @@ int __connman_device_request_hidden_scan(struct connman_device *device,
 				const char *identity, const char *passphrase,
 				gpointer user_data);
 
-connman_bool_t __connman_device_isfiltered(const char *devname);
+bool __connman_device_isfiltered(const char *devname);
 
 void __connman_device_set_network(struct connman_device *device,
 					struct connman_network *network);
@@ -495,17 +547,13 @@ int __connman_device_enable(struct connman_device *device);
 int __connman_device_disable(struct connman_device *device);
 int __connman_device_disconnect(struct connman_device *device);
 
-connman_bool_t __connman_device_has_driver(struct connman_device *device);
-
-void __connman_device_set_reconnect(struct connman_device *device,
-						connman_bool_t reconnect);
-connman_bool_t __connman_device_get_reconnect(struct connman_device *device);
+bool __connman_device_has_driver(struct connman_device *device);
 
 const char *__connman_device_get_type(struct connman_device *device);
 
 int __connman_rfkill_init(void);
 void __connman_rfkill_cleanup(void);
-int __connman_rfkill_block(enum connman_service_type type, connman_bool_t block);
+int __connman_rfkill_block(enum connman_service_type type, bool block);
 
 #include <connman/network.h>
 
@@ -526,16 +574,28 @@ int __connman_network_set_ipconfig(struct connman_network *network,
 const char *__connman_network_get_type(struct connman_network *network);
 const char *__connman_network_get_group(struct connman_network *network);
 const char *__connman_network_get_ident(struct connman_network *network);
-connman_bool_t __connman_network_get_weakness(struct connman_network *network);
+bool __connman_network_get_weakness(struct connman_network *network);
 
 int __connman_config_init();
 void __connman_config_cleanup(void);
 
-int __connman_config_load_service(GKeyFile *keyfile, const char *group, connman_bool_t persistent);
+int __connman_config_load_service(GKeyFile *keyfile, const char *group,
+				  bool persistent);
 int __connman_config_provision_service(struct connman_service *service);
 int __connman_config_provision_service_ident(struct connman_service *service,
 		const char *ident, const char *file, const char *entry);
 
+char *__connman_config_get_string(GKeyFile *key_file,
+	const char *group_name, const char *key, GError **error);
+
+char **__connman_config_get_string_list(GKeyFile *key_file,
+	const char *group_name, const char *key, gsize *length, GError **error);
+
+bool __connman_config_get_bool(GKeyFile *key_file,
+	const char *group_name, const char *key, GError **error);
+bool __connman_config_address_provisioned(const char *address,
+					const char *netmask);
+
 int __connman_tethering_init(void);
 void __connman_tethering_cleanup(void);
 
@@ -546,13 +606,17 @@ void __connman_tethering_set_disabled(void);
 int __connman_private_network_request(DBusMessage *msg, const char *owner);
 int __connman_private_network_release(const char *path);
 
+int __connman_ipv6pd_setup(const char *bridge);
+void __connman_ipv6pd_cleanup(void);
+
 #include <connman/provider.h>
 
-connman_bool_t __connman_provider_check_routes(struct connman_provider *provider);
+bool __connman_provider_check_routes(struct connman_provider *provider);
 int __connman_provider_append_user_route(struct connman_provider *provider,
 			int family, const char *network, const char *netmask);
 void __connman_provider_append_properties(struct connman_provider *provider, DBusMessageIter *iter);
 void __connman_provider_list(DBusMessageIter *iter, void *user_data);
+bool __connman_provider_is_immutable(struct connman_provider *provider);
 int __connman_provider_create_and_connect(DBusMessage *msg);
 const char * __connman_provider_get_ident(struct connman_provider *provider);
 int __connman_provider_indicate_state(struct connman_provider *provider,
@@ -587,29 +651,33 @@ struct connman_ipconfig *__connman_service_get_ip6config(
 				struct connman_service *service);
 struct connman_ipconfig *__connman_service_get_ipconfig(
 				struct connman_service *service, int family);
-connman_bool_t __connman_service_is_connected_state(struct connman_service *service,
+bool __connman_service_is_connected_state(struct connman_service *service,
 					enum connman_ipconfig_type type);
 const char *__connman_service_get_ident(struct connman_service *service);
 const char *__connman_service_get_path(struct connman_service *service);
+const char *__connman_service_get_name(struct connman_service *service);
 unsigned int __connman_service_get_order(struct connman_service *service);
+enum connman_service_state __connman_service_get_state(struct connman_service *service);
 void __connman_service_update_ordering(void);
 struct connman_network *__connman_service_get_network(struct connman_service *service);
 enum connman_service_security __connman_service_get_security(struct connman_service *service);
 const char *__connman_service_get_phase2(struct connman_service *service);
-connman_bool_t __connman_service_wps_enabled(struct connman_service *service);
+bool __connman_service_wps_enabled(struct connman_service *service);
 int __connman_service_set_favorite(struct connman_service *service,
-						connman_bool_t favorite);
+						bool favorite);
 int __connman_service_set_favorite_delayed(struct connman_service *service,
-					connman_bool_t favorite,
-					gboolean delay_ordering);
+					bool favorite,
+					bool delay_ordering);
 int __connman_service_set_immutable(struct connman_service *service,
-						connman_bool_t immutable);
+						bool immutable);
 int __connman_service_set_ignore(struct connman_service *service,
-						connman_bool_t ignore);
+						bool ignore);
 void __connman_service_set_userconnect(struct connman_service *service,
-						connman_bool_t userconnect);
+						bool userconnect);
 void __connman_service_set_search_domains(struct connman_service *service,
 					char **domains);
+void __connman_service_update_search_domains(struct connman_service *service,
+					char **domains);
 
 void __connman_service_set_string(struct connman_service *service,
 					const char *key, const char *value);
@@ -630,9 +698,10 @@ int __connman_service_indicate_default(struct connman_service *service);
 int __connman_service_connect(struct connman_service *service);
 int __connman_service_disconnect(struct connman_service *service);
 int __connman_service_disconnect_all(void);
+void __connman_service_set_active_session(bool enable, GSList *list);
 void __connman_service_auto_connect(void);
-gboolean __connman_service_remove(struct connman_service *service);
-connman_bool_t __connman_service_is_provider_pending(struct connman_service *service);
+bool __connman_service_remove(struct connman_service *service);
+bool __connman_service_is_provider_pending(struct connman_service *service);
 void __connman_service_set_provider_pending(struct connman_service *service,
 							DBusMessage *msg);
 void __connman_service_set_hidden_data(struct connman_service *service,
@@ -650,9 +719,9 @@ const char *__connman_service_type2string(enum connman_service_type type);
 enum connman_service_type __connman_service_string2type(const char *str);
 
 int __connman_service_nameserver_append(struct connman_service *service,
-				const char *nameserver, gboolean is_auto);
+				const char *nameserver, bool is_auto);
 int __connman_service_nameserver_remove(struct connman_service *service,
-				const char *nameserver, gboolean is_auto);
+				const char *nameserver, bool is_auto);
 void __connman_service_nameserver_clear(struct connman_service *service);
 void __connman_service_nameserver_add_routes(struct connman_service *service,
 						const char *gw);
@@ -668,13 +737,15 @@ void __connman_service_timeserver_changed(struct connman_service *service,
 		GSList *ts_list);
 void __connman_service_set_pac(struct connman_service *service,
 					const char *pac);
-connman_bool_t __connman_service_is_hidden(struct connman_service *service);
-connman_bool_t __connman_service_is_split_routing(struct connman_service *service);
+bool __connman_service_is_hidden(struct connman_service *service);
+bool __connman_service_is_split_routing(struct connman_service *service);
 int __connman_service_get_index(struct connman_service *service);
 void __connman_service_set_hidden(struct connman_service *service);
+void __connman_service_set_hostname(struct connman_service *service,
+						const char *hostname);
+const char *__connman_service_get_hostname(struct connman_service *service);
 void __connman_service_set_domainname(struct connman_service *service,
 						const char *domainname);
-const char *__connman_service_get_domainname(struct connman_service *service);
 const char *__connman_service_get_nameserver(struct connman_service *service);
 void __connman_service_set_proxy_autoconfig(struct connman_service *service,
 							const char *url);
@@ -703,21 +774,11 @@ void __connman_service_counter_unregister(const char *counter);
 
 #include <connman/session.h>
 
-struct service_entry;
-typedef connman_bool_t (* service_match_cb) (struct connman_session *session,
-					struct connman_service *service);
-typedef struct service_entry* (* create_service_entry_cb) (
-					struct connman_service *service,
-					const char *name,
-					enum connman_service_state state);
+typedef void (* service_iterate_cb) (struct connman_service *service,
+					void *user_data);
 
-GSequence *__connman_service_get_list(struct connman_session *session,
-				service_match_cb service_match,
-				create_service_entry_cb create_service_entry,
-				GDestroyNotify destroy_service_entry);
+int __connman_service_iterate_services(service_iterate_cb cb, void *user_data);
 
-void __connman_service_session_inc(struct connman_service *service);
-connman_bool_t __connman_service_session_dec(struct connman_service *service);
 void __connman_service_mark_dirty();
 void __connman_service_save(struct connman_service *service);
 
@@ -736,7 +797,7 @@ void __connman_notifier_enter_online(enum connman_service_type type);
 void __connman_notifier_leave_online(enum connman_service_type type);
 void __connman_notifier_connect(enum connman_service_type type);
 void __connman_notifier_disconnect(enum connman_service_type type);
-void __connman_notifier_offlinemode(connman_bool_t enabled);
+void __connman_notifier_offlinemode(bool enabled);
 void __connman_notifier_default_changed(struct connman_service *service);
 void __connman_notifier_proxy_changed(struct connman_service *service);
 void __connman_notifier_service_state_changed(struct connman_service *service,
@@ -744,7 +805,7 @@ void __connman_notifier_service_state_changed(struct connman_service *service,
 void __connman_notifier_ipconfig_changed(struct connman_service *service,
 					struct connman_ipconfig *ipconfig);
 
-connman_bool_t __connman_notifier_is_connected(void);
+bool __connman_notifier_is_connected(void);
 const char *__connman_notifier_get_state(void);
 
 #include <connman/rtnl.h>
@@ -759,8 +820,8 @@ unsigned int __connman_rtnl_update_interval_remove(unsigned int interval);
 int __connman_rtnl_request_update(void);
 int __connman_rtnl_send(const void *buf, size_t len);
 
-connman_bool_t __connman_session_mode();
-void __connman_session_set_mode(connman_bool_t enable);
+bool __connman_session_mode();
+void __connman_session_set_mode(bool enable);
 
 int __connman_session_create(DBusMessage *msg);
 int __connman_session_destroy(DBusMessage *msg);
@@ -785,10 +846,10 @@ void __connman_stats_cleanup(void);
 int __connman_stats_service_register(struct connman_service *service);
 void __connman_stats_service_unregister(struct connman_service *service);
 int  __connman_stats_update(struct connman_service *service,
-				connman_bool_t roaming,
+				bool roaming,
 				struct connman_stats_data *data);
 int __connman_stats_get(struct connman_service *service,
-				connman_bool_t roaming,
+				bool roaming,
 				struct connman_stats_data *data);
 
 int __connman_iptables_dump(const char *table_name);
@@ -870,8 +931,8 @@ void __connman_ippool_deladdr(int index, const char *address,
 
 int __connman_bridge_create(const char *name);
 int __connman_bridge_remove(const char *name);
-int __connman_bridge_enable(const char *name, const char *gateway,
-				const char *broadcast);
+int __connman_bridge_enable(const char *name, const char *ip_address,
+			int prefix_len, const char *broadcast);
 int __connman_bridge_disable(const char *name);
 
 int __connman_nat_init(void);
@@ -894,3 +955,37 @@ int __connman_firewall_disable(struct firewall_context *ctx);
 
 int __connman_firewall_init(void);
 void __connman_firewall_cleanup(void);
+
+typedef int (* connman_nfacct_flush_cb_t) (unsigned int error, void *user_data);
+
+int __connman_nfacct_flush(connman_nfacct_flush_cb_t cb, void *user_data);
+
+struct nfacct_context;
+
+typedef void (* connman_nfacct_enable_cb_t) (unsigned int error,
+						struct nfacct_context *ctx,
+						void *user_data);
+typedef void (* connman_nfacct_disable_cb_t) (unsigned int error,
+						struct nfacct_context *ctx,
+						void *user_data);
+typedef void (* connman_nfacct_stats_cb_t) (struct nfacct_context *ctx,
+						uint64_t packets,
+						uint64_t bytes,
+						void *user_data);
+
+struct nfacct_context *__connman_nfacct_create_context(void);
+void __connman_nfacct_destroy_context(struct nfacct_context *ctx);
+
+int __connman_nfacct_add(struct nfacct_context *ctx, const char *name,
+				connman_nfacct_stats_cb_t cb,
+				void *user_data);
+int __connman_nfacct_enable(struct nfacct_context *ctx,
+				connman_nfacct_enable_cb_t cb,
+				void *user_data);
+int __connman_nfacct_disable(struct nfacct_context *ctx,
+				connman_nfacct_disable_cb_t cb,
+				void *user_data);
+
+
+int __connman_nfacct_init(void);
+void __connman_nfacct_cleanup(void);
