@@ -54,7 +54,6 @@ struct connman_device {
 	bool powered;
 	bool scanning;
 	bool disconnected;
-	bool reconnect;
 	char *name;
 	char *node;
 	char *address;
@@ -236,7 +235,6 @@ int __connman_device_disable(struct connman_device *device)
 		return -EALREADY;
 
 	device->powered_pending = PENDING_DISABLE;
-	device->reconnect = false;
 
 	if (device->network) {
 		struct connman_service *service =
@@ -654,8 +652,6 @@ int connman_device_disconnect_service(struct connman_device *device)
 {
 	DBG("device %p", device);
 
-	device->reconnect = false;
-
 	if (device->network) {
 		struct connman_service *service =
 			connman_service_lookup_from_network(device->network);
@@ -673,8 +669,6 @@ int connman_device_reconnect_service(struct connman_device *device)
 {
 	DBG("device %p", device);
 
-	device->reconnect = true;
-
 	__connman_service_auto_connect();
 
 	return 0;
@@ -953,18 +947,6 @@ void __connman_device_set_network(struct connman_device *device,
 	}
 }
 
-void __connman_device_set_reconnect(struct connman_device *device,
-						bool reconnect)
-{
-	device->reconnect = reconnect;
-}
-
-bool  __connman_device_get_reconnect(
-				struct connman_device *device)
-{
-	return device->reconnect;
-}
-
 static bool match_driver(struct connman_device *device,
 					struct connman_device_driver *driver)
 {
@@ -1424,6 +1406,7 @@ static void cleanup_devices(void)
 	for (i = 0; interfaces[i]; i++) {
 		bool filtered;
 		int index;
+		struct sockaddr_in sin_addr, sin_mask;
 
 		filtered = __connman_device_isfiltered(interfaces[i]);
 		if (filtered)
@@ -1433,6 +1416,25 @@ static void cleanup_devices(void)
 		if (index < 0)
 			continue;
 
+		if (!__connman_inet_get_address_netmask(index, &sin_addr,
+							&sin_mask)) {
+			char *address = g_strdup(inet_ntoa(sin_addr.sin_addr));
+			char *netmask = g_strdup(inet_ntoa(sin_mask.sin_addr));
+
+			if (__connman_config_address_provisioned(address,
+								netmask)) {
+				DBG("Skip %s which is already provisioned "
+					"with %s/%s", interfaces[i], address,
+					netmask);
+				g_free(address);
+				g_free(netmask);
+				continue;
+			}
+
+			g_free(address);
+			g_free(netmask);
+		}
+
 		DBG("cleaning up %s index %d", interfaces[i], index);
 
 		connman_inet_ifdown(index);
