@@ -34,7 +34,6 @@
 
 static DBusConnection *connection;
 static GHashTable *session_hash;
-static bool sessionmode;
 static struct connman_session *ecall_session;
 static GSList *policy_list;
 static uint32_t session_mark = 256;
@@ -61,22 +60,9 @@ enum connman_session_state {
 	CONNMAN_SESSION_STATE_ONLINE         = 2,
 };
 
-struct service_entry {
-	struct connman_session *session;
-	/* track why this service was selected */
-	enum connman_session_reason reason;
-	enum connman_service_state state;
-	const char *name;
-	struct connman_service *service;
-	char *ifname;
-	const char *bearer;
-	GSList *pending_timeouts;
-};
-
 struct session_info {
 	struct connman_session_config config;
 	enum connman_session_state state;
-	struct service_entry *entry;
 	enum connman_session_reason reason;
 };
 
@@ -91,6 +77,8 @@ struct connman_session {
 	bool append_all;
 	struct session_info *info;
 	struct session_info *info_last;
+	struct connman_service *service;
+	struct connman_service *service_last;
 	struct connman_session_config *policy_config;
 	GSList *user_allowed_bearers;
 
@@ -399,14 +387,13 @@ static void del_default_route(struct connman_session *session)
 
 static void add_default_route(struct connman_session *session)
 {
-	struct session_info *info = session->info;
 	struct connman_ipconfig *ipconfig;
 	int err;
 
-	if (!info->entry)
+	if (!session->service)
 		return;
 
-	ipconfig = __connman_service_get_ip4config(info->entry->service);
+	ipconfig = __connman_service_get_ip4config(session->service);
 	session->index = __connman_ipconfig_get_index(ipconfig);
 	session->gateway = g_strdup(__connman_ipconfig_get_gateway(ipconfig));
 
@@ -474,16 +461,15 @@ static void cleanup_session_final(struct connman_session *session)
 
 	DBG("remove %s", session->session_path);
 
+	if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+		__connman_service_set_active_session(false,
+				session->info->config.allowed_bearers);
+
 	g_slist_free(session->user_allowed_bearers);
 	if (session->service_hash)
 		g_hash_table_destroy(session->service_hash);
 	g_list_free(session->service_list);
 
-	if (info->entry &&
-			info->entry->reason == CONNMAN_SESSION_REASON_CONNECT) {
-		__connman_service_disconnect(info->entry->service);
-	}
-
 	free_session(session);
 }
 
@@ -862,10 +848,12 @@ static void append_notify(DBusMessageIter *dict,
 	struct session_info *info = session->info;
 	struct session_info *info_last = session->info_last;
 	struct connman_service *service;
-	const char *name, *ifname, *bearer;
+	enum connman_service_type type;
+	const char *name, *bearer;
+	char *ifname;
+	int idx;
 
-	if (session->append_all ||
-			info->state != info_last->state) {
+	if (session->append_all || info->state != info_last->state) {
 		const char *state = state2string(info->state);
 
 		connman_dbus_dict_append_basic(dict, "State",
@@ -874,18 +862,23 @@ static void append_notify(DBusMessageIter *dict,
 		info_last->state = info->state;
 	}
 
-	if (session->append_all ||
-			info->entry != info_last->entry) {
-		if (!info->entry) {
-			name = "";
-			ifname = "";
-			service = NULL;
-			bearer = "";
+	if (session->append_all || session->service != session->service_last) {
+		if (session->service) {
+			service = session->service;
+			name = __connman_service_get_name(service);
+			idx = __connman_service_get_index(service);
+
+			ifname = connman_inet_ifname(idx);
+			if (!ifname)
+				ifname = g_strdup("");
+
+			type = connman_service_get_type(service);
+			bearer = service2bearer(type);
 		} else {
-			name = info->entry->name;
-			ifname = info->entry->ifname;
-			service = info->entry->service;
-			bearer = info->entry->bearer;
+			service = NULL;
+			name = "";
+			ifname = g_strdup("");
+			bearer = "";
 		}
 
 		connman_dbus_dict_append_basic(dict, "Name",
@@ -908,7 +901,9 @@ static void append_notify(DBusMessageIter *dict,
 						DBUS_TYPE_STRING,
 						&bearer);
 
-		info_last->entry = info->entry;
+		g_free(ifname);
+
+		session->service_last = session->service;
 	}
 
 	if (session->append_all ||
@@ -968,7 +963,7 @@ static bool compute_notifiable_changes(struct connman_session *session)
 	if (info->state != info_last->state)
 		return true;
 
-	if (info->entry != info_last->entry &&
+	if (session->service != session->service_last &&
 			info->state >= CONNMAN_SESSION_STATE_CONNECTED)
 		return true;
 
@@ -1011,20 +1006,16 @@ static gboolean session_notify(gpointer user_data)
 
 static void ipconfig_ipv4_changed(struct connman_session *session)
 {
-	struct session_info *info = session->info;
-
 	connman_dbus_setting_changed_dict(session->owner, session->notify_path,
 						"IPv4", append_ipconfig_ipv4,
-						info->entry->service);
+						session->service);
 }
 
 static void ipconfig_ipv6_changed(struct connman_session *session)
 {
-	struct session_info *info = session->info;
-
 	connman_dbus_setting_changed_dict(session->owner, session->notify_path,
 						"IPv6", append_ipconfig_ipv6,
-						info->entry->service);
+						session->service);
 }
 
 static bool service_type_match(struct connman_session *session,
@@ -1137,12 +1128,11 @@ static gint sort_allowed_bearers(struct connman_service *service_a,
 
 static gint sort_services(gconstpointer a, gconstpointer b, gpointer user_data)
 {
-	struct service_entry *entry_a = (void *)a;
-	struct service_entry *entry_b = (void *)b;
+	struct connman_service *service_a = (void *)a;
+	struct connman_service *service_b = (void *)b;
 	struct connman_session *session = user_data;
 
-	return sort_allowed_bearers(entry_a->service, entry_b->service,
-				session);
+	return sort_allowed_bearers(service_a, service_b, session);
 }
 
 static enum connman_session_state service_to_session_state(enum connman_service_state state)
@@ -1201,6 +1191,14 @@ static bool is_connecting(enum connman_service_state state)
 	return false;
 }
 
+static bool is_disconnected(enum connman_service_state state)
+{
+	if (is_connected(state) || is_connecting(state))
+		return false;
+
+	return true;
+}
+
 static bool explicit_connect(enum connman_session_reason reason)
 {
 	switch (reason) {
@@ -1214,228 +1212,105 @@ static bool explicit_connect(enum connman_session_reason reason)
 	return false;
 }
 
-static bool explicit_disconnect(struct session_info *info)
-{
-	if (!info->entry)
-		return false;
-
-	DBG("reason %s service %p state %d",
-		reason2string(info->entry->reason),
-		info->entry->service, info->entry->state);
-
-	if (info->entry->reason == CONNMAN_SESSION_REASON_UNKNOWN)
-		return false;
-
-	if (!explicit_connect(info->entry->reason))
-		return false;
-
-	if (!__connman_service_session_dec(info->entry->service))
-		return false;
-
-	return true;
-}
-
-struct pending_data {
-	unsigned int timeout;
-	struct service_entry *entry;
-	GSourceFunc cb;
-};
-
-static void pending_timeout_free(gpointer data, gpointer user_data)
-{
-	struct pending_data *pending = data;
-
-	DBG("pending %p timeout %d", pending, pending->timeout);
-	g_source_remove(pending->timeout);
-	g_free(pending);
-}
-
-static void pending_timeout_remove_all(struct service_entry *entry)
-{
-	DBG("");
-
-	g_slist_foreach(entry->pending_timeouts, pending_timeout_free, NULL);
-	g_slist_free(entry->pending_timeouts);
-	entry->pending_timeouts = NULL;
-}
-
-static gboolean pending_timeout_cb(gpointer data)
-{
-	struct pending_data *pending = data;
-	struct service_entry *entry = pending->entry;
-	bool ret;
-
-	DBG("pending %p timeout %d", pending, pending->timeout);
-
-	ret = pending->cb(pending->entry);
-	if (!ret) {
-		entry->pending_timeouts =
-			g_slist_remove(entry->pending_timeouts,
-					pending);
-		g_free(pending);
-	}
-	return ret;
-}
-
-static bool pending_timeout_add(unsigned int seconds,
-					GSourceFunc cb,
-					struct service_entry *entry)
-{
-	struct pending_data *pending = g_try_new0(struct pending_data, 1);
-
-	if (!pending || !cb || !entry) {
-		g_free(pending);
-		return false;
-	}
-
-	pending->cb = cb;
-	pending->entry = entry;
-	pending->timeout = g_timeout_add_seconds(seconds, pending_timeout_cb,
-						pending);
-	entry->pending_timeouts = g_slist_prepend(entry->pending_timeouts,
-						pending);
-
-	DBG("pending %p entry %p timeout id %d", pending, entry,
-		pending->timeout);
-
-	return true;
-}
-
-static gboolean call_disconnect(gpointer user_data)
-{
-	struct service_entry *entry = user_data;
-	struct connman_service *service = entry->service;
-
-	/*
-	 * TODO: We should mark this entry as pending work. In case
-	 * disconnect fails we just unassign this session from the
-	 * service and can't do anything later on it
-	 */
-	DBG("disconnect service %p", service);
-	__connman_service_disconnect(service);
-
-	return FALSE;
-}
-
-static gboolean call_connect(gpointer user_data)
-{
-	struct service_entry *entry = user_data;
-	struct connman_service *service = entry->service;
-
-	DBG("connect service %p", service);
-	__connman_service_connect(service);
-
-	return FALSE;
-}
-
-static void deselect_service(struct session_info *info)
-{
-	struct service_entry *entry;
-	bool disconnect, connected;
-
-	DBG("");
-
-	if (!info->entry)
-		return;
-
-	disconnect = explicit_disconnect(info);
-
-	connected = is_connecting(info->entry->state) ||
-			is_connected(info->entry->state);
-
-	info->state = CONNMAN_SESSION_STATE_DISCONNECTED;
-	info->entry->reason = CONNMAN_SESSION_REASON_UNKNOWN;
-
-	entry = info->entry;
-	info->entry = NULL;
-
-	DBG("disconnect %d connected %d", disconnect, connected);
-
-	if (disconnect && connected)
-		pending_timeout_add(0, call_disconnect, entry);
-}
-
 static void deselect_and_disconnect(struct connman_session *session)
 {
 	struct session_info *info = session->info;
 
-	deselect_service(info);
+	if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+		__connman_service_set_active_session(false,
+				info->config.allowed_bearers);
 
 	info->reason = CONNMAN_SESSION_REASON_FREE_RIDE;
 
+	info->state = CONNMAN_SESSION_STATE_DISCONNECTED;
+
+	if (!session->service)
+		return;
+
+	session->service = NULL;
+
 	update_routing_table(session);
 }
 
-static void select_connected_service(struct session_info *info,
-					struct service_entry *entry)
+static void select_connected_service(struct connman_session *session,
+					struct connman_service *service)
 {
+	struct session_info *info = session->info;
+	enum connman_service_state service_state;
 	enum connman_session_state state;
 
-	state = service_to_session_state(entry->state);
+	service_state = __connman_service_get_state(service);
+	state = service_to_session_state(service_state);
 	if (!is_type_matching_state(&state, info->config.type))
 		return;
 
 	info->state = state;
 
-	info->entry = entry;
-	info->entry->reason = info->reason;
+	session->service = service;
+}
+
+static void select_offline_service(struct connman_session *session,
+					struct connman_service *service)
+{
+	struct session_info *info = session->info;
+	enum connman_service_state service_state;
 
 	if (!explicit_connect(info->reason))
 		return;
 
-	__connman_service_session_inc(info->entry->service);
+	service_state = __connman_service_get_state(service);
+	info->state = service_to_session_state(service_state);
+
+	session->service = service;
 }
 
-static void select_offline_service(struct session_info *info,
-					struct service_entry *entry)
+static void select_service(struct connman_session *session,
+				struct connman_service *service)
 {
-	if (!explicit_connect(info->reason))
-		return;
+	enum connman_service_state service_state;
 
-	info->state = service_to_session_state(entry->state);
+	DBG("service %p", service);
 
-	info->entry = entry;
-	info->entry->reason = info->reason;
-
-	__connman_service_session_inc(info->entry->service);
-	pending_timeout_add(0, call_connect, entry);
-}
-
-static void select_service(struct session_info *info,
-				struct service_entry *entry)
-{
-	DBG("service %p", entry->service);
-
-	if (is_connected(entry->state))
-		select_connected_service(info, entry);
+	service_state = __connman_service_get_state(service);
+	if (is_connected(service_state))
+		select_connected_service(session, service);
 	else
-		select_offline_service(info, entry);
-
-	update_routing_table(entry->session);
+		select_offline_service(session, service);
 }
 
 static void select_and_connect(struct connman_session *session,
 				enum connman_session_reason reason)
 {
 	struct session_info *info = session->info;
-	struct service_entry *entry = NULL;
+	struct connman_service *service = NULL;
+	enum connman_service_state service_state;
 	GList *list;
 
 	DBG("session %p reason %s", session, reason2string(reason));
 
+	if (info->reason != reason &&
+			reason == CONNMAN_SESSION_REASON_CONNECT) {
+		__connman_service_set_active_session(true,
+				info->config.allowed_bearers);
+
+		__connman_service_auto_connect();
+	}
+
 	info->reason = reason;
 
 	for (list = session->service_list; list; list = list->next) {
-		entry = list->data;
+		service = list->data;
 
-		switch (entry->state) {
+		service_state = __connman_service_get_state(service);
+		switch (service_state) {
 		case CONNMAN_SERVICE_STATE_ASSOCIATION:
 		case CONNMAN_SERVICE_STATE_CONFIGURATION:
 		case CONNMAN_SERVICE_STATE_READY:
 		case CONNMAN_SERVICE_STATE_ONLINE:
 		case CONNMAN_SERVICE_STATE_IDLE:
 		case CONNMAN_SERVICE_STATE_DISCONNECT:
-			select_service(info, entry);
+			select_service(session, service);
+			update_routing_table(session);
 			return;
 		case CONNMAN_SERVICE_STATE_UNKNOWN:
 		case CONNMAN_SERVICE_STATE_FAILURE:
@@ -1444,74 +1319,20 @@ static void select_and_connect(struct connman_session *session,
 	}
 }
 
-static struct service_entry *create_service_entry(
-					struct connman_session * session,
-					struct connman_service *service,
-					const char *name,
-					enum connman_service_state state)
-{
-	struct service_entry *entry;
-	enum connman_service_type type;
-	int idx;
-
-	entry = g_try_new0(struct service_entry, 1);
-	if (!entry)
-		return entry;
-
-	entry->reason = CONNMAN_SESSION_REASON_UNKNOWN;
-	entry->state = state;
-	if (name)
-		entry->name = name;
-	else
-		entry->name = "";
-	entry->service = service;
-
-	idx = __connman_service_get_index(entry->service);
-	entry->ifname = connman_inet_ifname(idx);
-	if (!entry->ifname)
-		entry->ifname = g_strdup("");
-
-	type = connman_service_get_type(entry->service);
-	entry->bearer = service2bearer(type);
-
-	entry->session = session;
-
-	return entry;
-}
-
 static void iterate_service_cb(struct connman_service *service,
-				const char *name,
-				enum connman_service_state state,
 				void *user_data)
 {
 	struct connman_session *session = user_data;
-	struct service_entry *entry;
+	enum connman_service_state service_state;
+
+	service_state = __connman_service_get_state(service);
+	if (!is_connected(service_state))
+		return;
 
 	if (!service_match(session, service))
 		return;
 
-	entry = create_service_entry(session, service, name, state);
-	if (!entry)
-		return;
-
-	g_hash_table_replace(session->service_hash, service, entry);
-}
-
-static void destroy_service_entry(gpointer data)
-{
-	struct service_entry *entry = data;
-	struct session_info *info = entry->session->info;
-	struct connman_session *session;
-
-	if (info && info->entry == entry) {
-		session = entry->session;
-		deselect_and_disconnect(session);
-	}
-
-	pending_timeout_remove_all(entry);
-	g_free(entry->ifname);
-
-	g_free(entry);
+	g_hash_table_replace(session->service_hash, service, NULL);
 }
 
 static void populate_service_list(struct connman_session *session)
@@ -1521,19 +1342,19 @@ static void populate_service_list(struct connman_session *session)
 
 	session->service_hash =
 		g_hash_table_new_full(g_direct_hash, g_direct_equal,
-					NULL, destroy_service_entry);
+					NULL, NULL);
 	__connman_service_iterate_services(iterate_service_cb, session);
 
 	g_hash_table_iter_init(&iter, session->service_hash);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		struct service_entry *entry = value;
+		struct connman_service *service = key;
 
-		DBG("service %p type %s name %s", entry->service,
-			service2bearer(connman_service_get_type(entry->service)),
-			entry->name);
+		DBG("service %p type %s name %s", service,
+			service2bearer(connman_service_get_type(service)),
+			__connman_service_get_name(service));
 		session->service_list = g_list_prepend(session->service_list,
-							entry);
+							service);
 	}
 
 	session->service_list = g_list_sort_with_data(session->service_list,
@@ -1545,8 +1366,8 @@ static void session_changed(struct connman_session *session,
 {
 	struct session_info *info = session->info;
 	struct session_info *info_last = session->info_last;
-	struct connman_service *service;
-	struct service_entry *entry = NULL, *entry_last = NULL;
+	struct connman_service *service = NULL, *service_last = NULL;
+	enum connman_service_state service_state;
 	GHashTable *service_hash_last;
 
 	/*
@@ -1557,10 +1378,12 @@ static void session_changed(struct connman_session *session,
 	DBG("session %p trigger %s reason %s", session, trigger2string(trigger),
 						reason2string(info->reason));
 
-	if (info->entry) {
+
+	if (session->service) {
 		enum connman_session_state state;
 
-		state = service_to_session_state(info->entry->state);
+		service_state = __connman_service_get_state(session->service);
+		state = service_to_session_state(service_state);
 
 		if (is_type_matching_state(&state, info->config.type))
 			info->state = state;
@@ -1577,23 +1400,20 @@ static void session_changed(struct connman_session *session,
 			g_list_free(session->service_list);
 			session->service_list = NULL;
 
-			if (info->entry)
-				service = info->entry->service;
-			else
-				service = NULL;
+			service = session->service;
 
 			populate_service_list(session);
 
-			if (info->entry) {
-				entry_last = g_hash_table_lookup(
+			if (service) {
+				service_last = g_hash_table_lookup(
 							service_hash_last,
-							info->entry->service);
-				entry = g_hash_table_lookup(
-							session->service_hash,
 							service);
 			}
 
-			if (!entry && entry_last) {
+			if (service_last &&
+				!g_hash_table_lookup(session->service_hash,
+							service)) {
+
 				/*
 				 * The currently selected service is
 				 * not part of this session anymore.
@@ -1623,21 +1443,17 @@ static void session_changed(struct connman_session *session,
 		 * strategy.
 		 */
 	case CONNMAN_SESSION_TRIGGER_CONNECT:
-		if (info->state >= CONNMAN_SESSION_STATE_CONNECTED) {
-			if (info->entry->reason == CONNMAN_SESSION_REASON_CONNECT)
+		if (info->state >= CONNMAN_SESSION_STATE_CONNECTED)
+			break;
+
+		if (session->service) {
+			service_state = __connman_service_get_state(
+						session->service);
+			if (is_connecting(service_state))
 				break;
-			info->entry->reason = CONNMAN_SESSION_REASON_CONNECT;
-			__connman_service_session_inc(info->entry->service);
-			break;
 		}
 
-		if (info->entry &&
-				is_connecting(info->entry->state)) {
-			break;
-		}
-
-		select_and_connect(session,
-				CONNMAN_SESSION_REASON_CONNECT);
+		select_and_connect(session, CONNMAN_SESSION_REASON_CONNECT);
 
 		break;
 	case CONNMAN_SESSION_TRIGGER_DISCONNECT:
@@ -1645,15 +1461,15 @@ static void session_changed(struct connman_session *session,
 
 		break;
 	case CONNMAN_SESSION_TRIGGER_SERVICE:
-		if (info->entry &&
-			(is_connecting(info->entry->state) ||
-				is_connected(info->entry->state)))
+		if (session->service) {
+			service_state = __connman_service_get_state(
+						session->service);
+			if (is_connecting(service_state) ||
+				is_connected(service_state))
 			break;
+		}
 
-		deselect_and_disconnect(session);
-
-		if (info->reason == CONNMAN_SESSION_REASON_FREE_RIDE)
-			select_and_connect(session, info->reason);
+		select_and_connect(session, info->reason);
 
 		break;
 	}
@@ -1692,9 +1508,17 @@ int connman_session_config_update(struct connman_session *session)
 	if (err < 0)
 		return err;
 
+	if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+		__connman_service_set_active_session(false,
+				info->config.allowed_bearers);
+
 	g_slist_free(info->config.allowed_bearers);
 	info->config.allowed_bearers = allowed_bearers;
 
+	if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+		__connman_service_set_active_session(true,
+				info->config.allowed_bearers);
+
 	info->config.type = apply_policy_on_type(
 				session->policy_config->type,
 				info->config.type);
@@ -1783,6 +1607,10 @@ static DBusMessage *change_session(DBusConnection *conn,
 			if (err < 0)
 				return __connman_error_failed(msg, -err);
 
+			if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+				__connman_service_set_active_session(false,
+						info->config.allowed_bearers);
+
 			g_slist_free(info->config.allowed_bearers);
 			session->user_allowed_bearers = allowed_bearers;
 
@@ -1793,6 +1621,11 @@ static DBusMessage *change_session(DBusConnection *conn,
 
 			if (err < 0)
 				return __connman_error_failed(msg, -err);
+
+			if (info->reason == CONNMAN_SESSION_REASON_CONNECT)
+				__connman_service_set_active_session(true,
+						info->config.allowed_bearers);
+
 		} else {
 			goto err;
 		}
@@ -1928,7 +1761,6 @@ static int session_create_final(struct creation_data *creation_data,
 				creation_data->type);
 	info->config.priority = session->policy_config->priority;
 	info->config.roaming_policy = session->policy_config->roaming_policy;
-	info->entry = NULL;
 
 	session->user_allowed_bearers = creation_data->allowed_bearers;
 	creation_data->allowed_bearers = NULL;
@@ -1965,7 +1797,6 @@ static int session_create_final(struct creation_data *creation_data,
 	info_last->state = info->state;
 	info_last->config.priority = info->config.priority;
 	info_last->config.roaming_policy = info->config.roaming_policy;
-	info_last->entry = info->entry;
 	info_last->config.allowed_bearers = info->config.allowed_bearers;
 
 	session->append_all = true;
@@ -2280,37 +2111,11 @@ int __connman_session_destroy(DBusMessage *msg)
 	return 0;
 }
 
-bool __connman_session_mode()
-{
-	return sessionmode;
-}
-
-void __connman_session_set_mode(bool enable)
-{
-	dbus_bool_t mode;
-
-	DBG("enable %d", enable);
-
-	if (sessionmode != enable) {
-		sessionmode = enable;
-
-		mode = sessionmode;
-		connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "SessionMode",
-				DBUS_TYPE_BOOLEAN, &mode);
-	}
-
-	if (sessionmode)
-		__connman_service_disconnect_all();
-}
-
-static void service_add(struct connman_service *service,
-			const char *name)
+static void service_add(struct connman_service *service, const char *name)
 {
 	GHashTableIter iter;
 	gpointer key, value;
 	struct connman_session *session;
-	struct service_entry *entry;
 
 	DBG("service %p", service);
 
@@ -2322,15 +2127,19 @@ static void service_add(struct connman_service *service,
 		if (!service_match(session, service))
 			continue;
 
-		entry = create_service_entry(session, service, name,
-						CONNMAN_SERVICE_STATE_IDLE);
-		if (!entry)
-			continue;
+		if (g_hash_table_lookup(session->service_hash, service)) {
+			session->service_list =
+				g_list_sort_with_data(session->service_list,
+						sort_services, session);
+		} else {
+			session->service_list =
+				g_list_insert_sorted_with_data(
+				session->service_list, service,
+				sort_services, session);
 
-		session->service_list = g_list_insert_sorted_with_data(
-			session->service_list, entry, sort_services, session);
-
-		g_hash_table_replace(session->service_hash, service, entry);
+			g_hash_table_replace(session->service_hash,
+						service, service);
+		}
 
 		session_changed(session, CONNMAN_SESSION_TRIGGER_SERVICE);
 	}
@@ -2338,32 +2147,26 @@ static void service_add(struct connman_service *service,
 
 static void service_remove(struct connman_service *service)
 {
-
 	GHashTableIter iter;
 	gpointer key, value;
-	struct connman_session *session;
-	struct session_info *info;
 
 	DBG("service %p", service);
 
 	g_hash_table_iter_init(&iter, session_hash);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		struct service_entry *entry;
-		session = value;
-		info = session->info;
+		struct connman_session *session = value;
 
-		entry = g_hash_table_lookup(session->service_hash, service);
-		if (!entry)
+		if (!g_hash_table_lookup(session->service_hash, service))
 			continue;
 
 		session->service_list = g_list_remove(session->service_list,
-							entry);
+							service);
 
 		g_hash_table_remove(session->service_hash, service);
 
-		if (info->entry && info->entry->service == service)
-			info->entry = NULL;
+		if (session->service && session->service == service)
+			session->service = NULL;
 		session_changed(session, CONNMAN_SESSION_TRIGGER_SERVICE);
 	}
 }
@@ -2371,24 +2174,12 @@ static void service_remove(struct connman_service *service)
 static void service_state_changed(struct connman_service *service,
 					enum connman_service_state state)
 {
-	GHashTableIter iter;
-	gpointer key, value;
-
 	DBG("service %p state %d", service, state);
 
-	g_hash_table_iter_init(&iter, session_hash);
-
-	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		struct connman_session *session = value;
-		struct service_entry *entry;
-
-		entry = g_hash_table_lookup(session->service_hash, service);
-		if (entry)
-			entry->state = state;
-
-		session_changed(session,
-				CONNMAN_SESSION_TRIGGER_SERVICE);
-	}
+	if (is_connected(state))
+		service_add(service, __connman_service_get_name(service));
+	else if (is_disconnected(state))
+		service_remove(service);
 }
 
 static void ipconfig_changed(struct connman_service *service,
@@ -2413,7 +2204,7 @@ static void ipconfig_changed(struct connman_service *service,
 		if (info->state == CONNMAN_SESSION_STATE_DISCONNECTED)
 			continue;
 
-		if (info->entry && info->entry->service == service) {
+		if (session->service && session->service == service) {
 			if (type == CONNMAN_IPCONFIG_TYPE_IPV4)
 				ipconfig_ipv4_changed(session);
 			else if (type == CONNMAN_IPCONFIG_TYPE_IPV6)
@@ -2424,8 +2215,6 @@ static void ipconfig_changed(struct connman_service *service,
 
 static struct connman_notifier session_notifier = {
 	.name			= "session",
-	.service_add		= service_add,
-	.service_remove		= service_remove,
 	.service_state_changed	= service_state_changed,
 	.ipconfig_changed	= ipconfig_changed,
 };
@@ -2464,8 +2253,6 @@ int __connman_session_init(void)
 	session_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
 						NULL, cleanup_session);
 
-	sessionmode = false;
-
 	__connman_nfacct_flush(session_nfacct_flush_cb, NULL);
 
 	return 0;
